/* $Id: api.c 13 2023-05-06 01:14:42Z nishi $ */
#include <nibbler/api.h>

#include <nibbler/config.h>
#include <nibbler/db.h>
#include <nibbler/sha512.h>
#include <nibbler/util.h>
#include <nibbler.h>

#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <dirent.h>
#include <sys/stat.h>
#include <time.h>

extern char* nibbler_body;

void nibbler_api(struct nibbler_config* config){
	char* path = getenv("PATH_REPLACED");
	printf("Content-Type: application/json\r\n");
	if(strcmp(path, "/v1/login") == 0){
		int i;
		int index = -1;
		for(i = 0; nibbler_body[i] != 0; i++){
			if(nibbler_body[i] == ':'){
				nibbler_body[i] = 0;
				index = i;
				break;
			}
		}
		if(index == -1){
			printf("Status: 400 Bad Request\r\n");
			printf("\r\n");
			printf("{\"error\": \"Required parameters missing, `username:password`\", \"code\": %d}", NIBBLER_CODE_PARAM_MISSING);
		}else{
			char* username = nibbler_body;
			char* password = nibbler_body + index + 1;
			datum key;
			key.dptr = username;
			key.dsize = strlen(username) + 1;
			datum value;
			if((value = dbm_fetch(nibbler_user_db, key)).dptr == NULL){
				printf("Status: 404 Not Found\r\n");
				printf("\r\n");
				printf("{\"error\": \"User not found\", \"code\": %d}", NIBBLER_CODE_USER_NOT_FOUND);
			}else{
				struct nibbler_user* user = (struct nibbler_user*)value.dptr;
				unsigned char* sha512d = nibbler_sha512(password);
				if(memcmp(sha512d, user->password, SHA512_DIGEST_LENGTH) == 0){
					int i = 0;
					const char* hex = "0123456789abcdef";
					FILE* rand = fopen("/dev/urandom", "r");
					char ch = 0;
					char* token = malloc(65);
					token[64] = 0;
regen:
					for(i = 0; i < 64; i++){
						fread(&ch, 1, 1, rand);
						token[i] = hex[ch & 0x0f];
					}
					key.dptr = token;
					key.dsize = strlen(token) + 1;
					if(dbm_fetch(nibbler_token_db, key).dptr != NULL) goto regen;
					fclose(rand);
					datum value;
					value.dptr = username;
					value.dsize = strlen(username) + 1;
					dbm_store(nibbler_token_db, key, value, DBM_INSERT);
					nibbler_init_db();
					printf("Status: 200 OK\r\n");
					printf("\r\n");
					printf("{\"message\": \"Login successful\", \"token\": \"%s\", \"code\": %d}", token, NIBBLER_CODE_SUCCESS);
					free(token);
				}else{
					printf("Status: 403 Forbidden\r\n");
					printf("\r\n");
					printf("{\"error\": \"Password incorrect\", \"code\": %d}", NIBBLER_CODE_PASSWORD_INCORRECT);
				}
				free(sha512d);
			}
		}
	}else if(strcmp(path, "/v1/signup") == 0){
		if(config->signup != 1){
			printf("Status: 403 Forbidden\r\n");
			printf("\r\n");
			printf("{\"error\": \"Signup not allowed\", \"code\": %d}", NIBBLER_CODE_SIGNUP_NOT_ALLOWED);
		}else{
			int i;
			int index = -1;
			for(i = 0; nibbler_body[i] != 0; i++){
				if(nibbler_body[i] == ':'){
					nibbler_body[i] = 0;
					index = i;
					break;
				}
			}
			if(index == -1){
				printf("Status: 400 Bad Request\r\n");
				printf("\r\n");
				printf("{\"error\": \"Required parameters missing, `username:password`\", \"code\": %d}", NIBBLER_CODE_PARAM_MISSING);
			}else{
				char* username = nibbler_body;
				char* password = nibbler_body + index + 1;
				datum key;
				key.dptr = username;
				key.dsize = strlen(username) + 1;
				if(dbm_fetch(nibbler_user_db, key).dptr == NULL){
					struct nibbler_user user;
					unsigned char* sha512d = nibbler_sha512(password);
					memcpy(user.password, sha512d, SHA512_DIGEST_LENGTH);
					free(sha512d);
					user.mod = 0;
					datum value;
					value.dptr = (void*)&user;
					value.dsize = sizeof(user);
					dbm_store(nibbler_user_db, key, value, DBM_INSERT);
					nibbler_init_db();
					printf("Status: 200 OK\r\n");
					printf("\r\n");
					printf("{\"message\": \"User successfully added\", \"code\": %d}", NIBBLER_CODE_SUCCESS);
				}else{
					printf("Status: 409 Conflict\r\n");
					printf("\r\n");
					printf("{\"error\": \"User already exists\", \"code\": %d}", NIBBLER_CODE_USER_ALREADY_EXISTS);
				}
			}
		}
	}else if(strcmp(path, "/v1/valid") == 0){
		datum key;
		key.dptr = nibbler_body;
		key.dsize = strlen(nibbler_body) + 1;
		datum value;
		if((value = dbm_fetch(nibbler_token_db, key)).dptr != NULL){
			printf("Status: 200 OK\r\n");
			printf("\r\n");
			printf("{\"username\": \"%s\", \"code\": %d}", value.dptr, NIBBLER_CODE_SUCCESS);
		}else{
			printf("Status: 401 Unauthorized\r\n");
			printf("\r\n");
			printf("{\"error\": \"Invalid token\", \"code\": %d}", NIBBLER_CODE_INVALID_TOKEN);
		}
	}else if(strcmp(path, "/v1/logout") == 0){
		datum key;
		key.dptr = nibbler_body;
		key.dsize = strlen(nibbler_body) + 1;
		datum value;
		if((value = dbm_fetch(nibbler_token_db, key)).dptr != NULL){
			dbm_delete(nibbler_token_db, key);
			printf("Status: 200 OK\r\n");
			printf("\r\n");
			printf("{\"message\": \"Logout successful\", \"code\": %d}", NIBBLER_CODE_SUCCESS);
			nibbler_init_db();
		}else{
			printf("Status: 401 Unauthorized\r\n");
			printf("\r\n");
			printf("{\"error\": \"Invalid token\", \"code\": %d}", NIBBLER_CODE_INVALID_TOKEN);
		}
	}else if(strcmp(path, "/v1/genre/list") == 0){
		printf("Status: 200 OK\r\n");
		printf("\r\n");
		printf("{\"code\": %d, \"genres\": {", NIBBLER_CODE_SUCCESS);
		DIR* dir = opendir(NIBBLER_GENRE_DB);
		struct dirent* d;
		int count = 0;
		while((d = readdir(dir)) != NULL){
			if(strcmp(d->d_name, ".") == 0 || strcmp(d->d_name, "..") == 0) continue;
			if(count != 0) printf(",");
			char* path = sstrcat3(NIBBLER_GENRE_DB "/", d->d_name, "/description");
			struct stat s;
			stat(path, &s);
			char* description = malloc(s.st_size + 1);
			description[s.st_size] = 0;
			FILE* f = fopen(path, "r");
			fread(description, 1, s.st_size, f);
			fclose(f);
			printf("\"%s\": \"%s\"", d->d_name, description);
			free(description);
			free(path);
			count++;
		}
		closedir(dir);
		printf("}}\n");
	}else if(strcmp(path, "/v1/thread/new") == 0){
		mkdir(NIBBLER_THREAD_DB, 0755);
		int i;
		int index = -1;
		int index2 = -1;
		for(i = 0; nibbler_body[i] != 0; i++){
			if(nibbler_body[i] == ':'){
				nibbler_body[i] = 0;
				if(index == -1){
					index = i;
				}else if(index2 == -1){
					index2 = i;
					break;
				}
			}
		}
		if(index == -1 || index2 == -1){
			printf("Status: 400 Bad Request\r\n");
			printf("\r\n");
			printf("{\"error\": \"Required parameters missing, `token:genre-name:thread-name`\", \"code\": %d}", NIBBLER_CODE_PARAM_MISSING);
		}else{
			char* token = nibbler_body;
			char* name = nibbler_body + index + 1;
			char* tname = nibbler_body + index2 + 1;
			if(strlen(tname) > 1024){
				printf("Status: 400 Bad Request\r\n");
				printf("\r\n");
				printf("{\"error\": \"Thread name length cannot be over 1024\", \"code\": %d}", NIBBLER_CODE_THREAD_NAME_OVER_1024);
				return;
			}
			datum key;
			key.dptr = token;
			key.dsize = strlen(token) + 1;
			if(dbm_fetch(nibbler_token_db, key).dptr != NULL){
				char* path = sstrcat(NIBBLER_GENRE_DB "/", name);
				struct stat s;
				if(stat(path, &s) == 0){
					char* dbpath = sstrcat(path, "/db");
					DBM* db = nibbler_open_db(dbpath);
					key.dptr = strdup("length");
					key.dsize = 7;
					datum value = dbm_fetch(db, key);
					uint64_t length = ((struct nibbler_length*)value.dptr)->length;
					datum tkey;
					tkey.dptr = (void*)&length;
					tkey.dsize = sizeof(length);
					datum tvalue;
					struct nibbler_thread thr;
					thr.type = DB_TYPE_THREAD;
					thr.id = length;
					memset(thr.title, 0, 1025);
					strcpy(thr.title, tname);
					tvalue.dptr = (void*)&thr;
					tvalue.dsize = sizeof(thr);
					dbm_store(db, tkey, tvalue, DBM_INSERT);
					struct nibbler_length s_length;
					s_length.type = DB_TYPE_LENGTH;
					s_length.length = length + 1;
					value.dptr = (void*)&s_length;
					value.dsize = sizeof(s_length);
					dbm_store(db, key, value, DBM_REPLACE);
					free(key.dptr);
					dbm_close(db);
					free(dbpath);
					char* tmp = sstrcat3(NIBBLER_THREAD_DB "/", name, ":");
					dbpath = malloc(1025);
					memset(dbpath, 0, 1025);
					strcpy(dbpath, tmp);
					sprintf(dbpath + strlen(tmp), "%lu", length);
					free(tmp);
					DBM* tdb = nibbler_open_db(dbpath);
					key.dptr = strdup("length");
					key.dsize = 7;
					struct nibbler_length tlen;
					tlen.type = DB_TYPE_LENGTH;
					tlen.length = 0;
					datum tdvalue;
					tdvalue.dptr = (void*)&tlen;
					tdvalue.dsize = sizeof(tlen);
					dbm_store(tdb, key, tdvalue, DBM_INSERT);
					free(key.dptr);
					key.dptr = strdup("name");
					key.dsize = 5;
					tdvalue.dptr = (void*)tname;
					tdvalue.dsize = strlen(tname) + 1;
					dbm_store(tdb, key, tdvalue, DBM_INSERT);
					dbm_close(tdb);
					free(key.dptr);
					free(dbpath);
					printf("Status: 200 OK\r\n");
					printf("\r\n");
					printf("{\"id\": \"%s:%lu\", \"message\": \"Thread created successfully\", \"code\": %d}", name, length, NIBBLER_CODE_SUCCESS);
				}else{
					printf("Status: 404 Not Found\r\n");
					printf("\r\n");
					printf("{\"error\": \"Genre not found\", \"code\": %d}", NIBBLER_CODE_GENRE_NOT_FOUND);
				}
				free(path);
			}else{
				printf("Status: 401 Unauthorized\r\n");
				printf("\r\n");
				printf("{\"error\": \"Invalid token\", \"code\": %d}", NIBBLER_CODE_INVALID_TOKEN);
			}
		}
	}else if(strcmp(path, "/v1/thread/send") == 0){
		int i;
		int index = -1;
		int index2 = -1;
		int index3 = -1;
		for(i = 0; nibbler_body[i] != 0; i++){
			if(nibbler_body[i] == ':'){
				nibbler_body[i] = 0;
				if(index == -1){
					index = i;
				}else if(index2 == -1){
					index2 = i;
				}else if(index3 == -1){
					index3 = i;
					break;
				}
			}
		}
		if(index == -1 || index2 == -1 || index3 == -1){
			printf("Status: 400 Bad Request\r\n");
			printf("\r\n");
			printf("{\"error\": \"Required parameters missing, `token:genre-name:thread-id:message`\", \"code\": %d}", NIBBLER_CODE_PARAM_MISSING);
		}else{
			char* token = nibbler_body;
			char* genre = nibbler_body + index + 1;
			char* thread = nibbler_body + index2 + 1;
			char* message = nibbler_body + index3 + 1;
			datum key;
			key.dptr = token;
			key.dsize = strlen(token) + 1;
			datum user;
			if((user = dbm_fetch(nibbler_token_db, key)).dptr == NULL){
				printf("Status: 401 Unauthorized\r\n");
				printf("\r\n");
				printf("{\"error\": \"Invalid token\", \"code\": %d}", NIBBLER_CODE_INVALID_TOKEN);
			}else{
				char* path = sstrcat4(NIBBLER_THREAD_DB "/", genre, ":", thread);
				DBM* db = nibbler_check_db(path);
				if(db == NULL){
					printf("Status: 404 Not Found\r\n");
					printf("\r\n");
					printf("{\"error\": \"Thread not found\", \"code\": %d}", NIBBLER_CODE_THREAD_NOT_FOUND);
				}else{
					key.dptr = strdup("length");
					key.dsize = 7;
					uint64_t length = ((struct nibbler_length*)dbm_fetch(db, key).dptr)->length;
					struct nibbler_message msg;
					msg.id = length;
					msg.type = DB_TYPE_MESSAGE;
					msg.time = time(NULL);
					length++;
					struct nibbler_length len;
					len.type = DB_TYPE_LENGTH;
					len.length = length;
					free(key.dptr);
					uint64_t writethis = length - 1;
					key.dptr = (void*)&writethis;
					key.dsize = sizeof(length);
					datum value;
					value.dptr = (void*)&msg;
					value.dsize = sizeof(msg);
					dbm_store(db, key, value, DBM_INSERT);
					key.dptr = strdup("length");
					key.dsize = 7;
					value.dptr = (void*)&len;
					value.dsize = sizeof(len);
					dbm_store(db, key, value, DBM_REPLACE);

					free(key.dptr);
					char* keyname = malloc(1025);
					memset(keyname, 0, 1025);
					sprintf(keyname, "%lu-user", writethis);
					key.dptr = keyname;
					key.dsize = strlen(keyname) + 1;
					value.dptr = user.dptr;
					value.dsize = strlen(user.dptr) + 1;
					dbm_store(db, key, value, DBM_INSERT);

					free(key.dptr);
					keyname = malloc(1025);
					memset(keyname, 0, 1025);
					sprintf(keyname, "%lu-message", writethis);
					key.dptr = keyname;
					key.dsize = strlen(keyname) + 1;
					value.dptr = message;
					value.dsize = strlen(message) + 1;
					dbm_store(db, key, value, DBM_INSERT);


					printf("Status: 200 OK\r\n");
					printf("\r\n");
					printf("{\"message\": \"Message got received\", \"code\": %d}", NIBBLER_CODE_SUCCESS);
					dbm_close(db);
					free(key.dptr);
				}
				free(path);
			}
		}
	}
}
