/* $Id: config.c 5 2023-05-05 09:48:02Z nishi $ */
#include <nibbler/config.h>

#include <nibbler.h>
#include <nibbler/util.h>
#include <nibbler/db.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

struct nibbler_config* nibbler_parse_config(const char* path){
	struct nibbler_config* config = malloc(sizeof(*config));
	FILE* f = fopen(path, "r");
	config->signup = 0;
	config->itworks = 1;
	config->title = strdup("Nibbler");
	config->csspath = strdup("/nibbler.css");
	config->iconpath = strdup("/icon.png");
	config->scriptpath = strdup("/nibbler.js");
	config->welcome = NULL;
	if(f != NULL){
		char* linebuf = malloc(1);
		linebuf[0] = 0;
		char* buf = malloc(2);
		buf[1] = 0;
		while(1){
			fread(buf, 1, 1, f);
			if(feof(f) || buf[0] == '\n'){
				if(strlen(linebuf) != 0){
					int index = -1;
					int i;
					for(i = 0; linebuf[i] != 0; i++){
						if(linebuf[i] == '='){
							index = i;
							linebuf[i] = 0;
							break;
						}
					}
					if(index != -1){
						char* key = linebuf;
						char* value = linebuf + index + 1;
						if(strcmp(key, "Signup") == 0){
							config->signup = strcmp(value, "0") == 0 ? 0 : 1;
						}else if(strcmp(key, "ItWorks") == 0){
							config->itworks = strcmp(value, "0") == 0 ? 0 : 1;
						}else if(strcmp(key, "Title") == 0){
							free(config->title);
							config->title = strdup(value);
						}else if(strcmp(key, "CSSPath") == 0){
							free(config->csspath);
							config->csspath = strdup(value);
						}else if(strcmp(key, "IconPath") == 0){
							free(config->iconpath);
							config->iconpath = strdup(value);
						}else if(strcmp(key, "ScriptPath") == 0){
							free(config->scriptpath);
							config->scriptpath = strdup(value);
						}else if(strcmp(key, "Welcome") == 0){
							if(config->welcome != NULL) free(config->welcome);
							config->welcome = strdup(value);
						}
					}
				}
				free(linebuf);
				linebuf = malloc(1);
				linebuf[0] = 0;
				if(feof(f)) break;
			}else{
				char* tmp = linebuf;
				linebuf = sstrcat(linebuf, buf);
				free(tmp);
			}
		}
		free(buf);
		free(linebuf);
		fclose(f);
	}
	return config;
}

void nibbler_free_config(struct nibbler_config* config){
	free(config->title);
	free(config->iconpath);
	free(config->csspath);
	free(config->scriptpath);
	if(config->welcome != NULL) free(config->welcome);
	free(config);
}

void nibbler_read_genres(struct nibbler_config* config){
	mkdir(NIBBLER_GENRE_DB, 0755);
	FILE* f = fopen(NIBBLER_GENRE, "r");
	if(f != NULL){
		char* linebuf = malloc(1);
		linebuf[0] = 0;
		char* buf = malloc(2);
		buf[1] = 0;
		while(1){
			fread(buf, 1, 1, f);
			if(feof(f) || buf[0] == '\n'){
				if(strlen(linebuf) != 0){
					int index = -1;
					int i;
					for(i = 0; linebuf[i] != 0; i++){
						if(linebuf[i] == ':'){
							linebuf[i] = 0;
							index = i;
							break;
						}
					}
					char* name = linebuf;
					if(index != -1){
						char* path = sstrcat(NIBBLER_GENRE_DB "/", name);
						mkdir(path, 0755);
						char* description = sstrcat(path, "/description");
						FILE* fdesc = fopen(description, "w");
						fwrite(linebuf + index + 1, strlen(linebuf + index + 1), 1, fdesc);
						fclose(fdesc);
						free(description);
						free(path);
					}
				}
				free(linebuf);
				linebuf = malloc(1);
				linebuf[0] = 0;
				if(feof(f)) break;
			}else{
				char* tmp = linebuf;
				linebuf = sstrcat(linebuf, buf);
				free(tmp);
			}
		}
		free(buf);
		free(linebuf);
		fclose(f);
	}
}
