/* $Id: db.c 9 2023-05-05 10:08:29Z nishi $ */
#define DB_SRC
#include <nibbler/db.h>

#include <nibbler.h>
#include <nibbler/util.h>

#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/file.h>

#ifndef O_EXLOCK
#define O_EXLOCK 0
#define FLOCK_NEED
#endif

DBM* nibbler_user_db = NULL;
DBM* nibbler_token_db = NULL;

DBM* nibbler_open_db(const char* path){
	DBM* result = NULL;
retry_opening:
	result = dbm_open(
		path,
		O_RDWR | O_CREAT | O_EXLOCK,
		S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH
	);
	if(result == NULL){
		usleep(100000);
		goto retry_opening;
	}
#ifdef FLOCK_NEED
	flock(result->dirfd, LOCK_EX);
#endif
	return result;
}

DBM* nibbler_check_db(const char* path){
	DBM* result = NULL;
	result = dbm_open(
		path,
		O_RDWR | O_EXLOCK,
		S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH
	);
#ifdef FLOCK_NEED
	flock(result->dirfd, LOCK_EX);
#endif

	return result;
}

void nibbler_init_db(){
	mkdir(NIBBLER_DATABASE, 0775);
	if(nibbler_user_db != NULL){
		dbm_close(nibbler_user_db);
		nibbler_user_db = NULL;
	}
	if(nibbler_token_db != NULL){
		dbm_close(nibbler_token_db);
		nibbler_token_db = NULL;
	}
retry_db:
	if(nibbler_user_db == NULL){
		nibbler_user_db = dbm_open(
			NIBBLER_USER_DB,
			O_RDWR | O_CREAT | O_EXLOCK,
			S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH
		);
	}
	if(nibbler_token_db == NULL){
		nibbler_token_db = dbm_open(
			NIBBLER_TOKEN_DB,
			O_RDWR | O_CREAT | O_EXLOCK,
			S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH
		);
	}
	if(nibbler_user_db == NULL || nibbler_token_db == NULL){
		usleep(100000);
		goto retry_db;
	}
#ifdef FLOCK_NEED
	flock(nibbler_user_db->dirfd, LOCK_EX);
#endif
#ifdef FLOCK_NEED
	flock(nibbler_token_db->dirfd, LOCK_EX);
#endif

}
