/* $Id: page.c 15 2023-05-07 04:45:42Z nishi $ */
#include <nibbler/page.h>

#include <nibbler/language.h>
#include <nibbler/db.h>
#include <nibbler/util.h>
#include <nibbler/api.h>

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <time.h>
#include <dirent.h>
#include <sys/stat.h>
#include <wchar.h>
#include <locale.h>

void nibbler_page(struct nibbler_config* config){
	setlocale(LC_CTYPE, "");
	if(strcmp(getenv("FIRST_PATH"), "/v1") == 0){
		nibbler_api(config);
		return;
	}
	char thread_exists = 0;
	char* date = malloc(128);
	time_t t = time(NULL);
	struct tm *time_st = localtime(&t);
	strftime(date, 128, "%s", time_st);
	if(!(	strcmp(getenv("PATH_REPLACED"), "/") == 0
	||	strcmp(getenv("PATH_REPLACED"), "/login") == 0
	||	strcmp(getenv("PATH_REPLACED"), "/signup") == 0
	||	strcmp(getenv("PATH_REPLACED"), "/new-thread") == 0
	||	strcmp(getenv("FIRST_PATH"), "/id") == 0
	)){
		printf("Status: 404 Not Found\r\n");
	}else{
		if(strcmp(getenv("FIRST_PATH"), "/id") == 0){
			char* path = sstrcat(NIBBLER_THREAD_DB "/", getenv("PATH_REPLACED") + strlen(getenv("FIRST_PATH")) + 1);
			struct stat s;
			DBM* db = nibbler_check_db(path);
			if(db != NULL){
				printf("Status: 200 OK\r\n");
				dbm_close(db);
				thread_exists = 1;
			}else{
				printf("Status: 404 Not Found\r\n");
			}
			free(path);
		}else{
			printf("Status: 200 OK\r\n");
		}
	}
	printf("Content-Type: text/html\r\n");
	printf("\r\n");
	printf("<html>\n");
	printf("	<head>\n");
	printf("		<meta charset=\"UTF-8\">\n");
	printf("		<title>\n");
	if(strcmp(getenv("PATH_REPLACED"), "/") == 0){
		NIBBLER_WELCOME_TITLE;
	}else if(strcmp(getenv("PATH_REPLACED"), "/login") == 0){
		printf("%s - " NIBBLER_LOGIN, config->title);
	}else if(strcmp(getenv("PATH_REPLACED"), "/signup") == 0){
		printf("%s - " NIBBLER_SIGNUP, config->title);
	}else if(strcmp(getenv("FIRST_PATH"), "/id") == 0){
		char* path = sstrcat(NIBBLER_THREAD_DB "/", getenv("PATH_REPLACED") + strlen(getenv("FIRST_PATH")) + 1);
		struct stat s;
		DBM* db = nibbler_check_db(path);
		if(db != NULL){
			datum key;
			key.dptr = strdup("name");
			key.dsize = 5;
			datum value = dbm_fetch(db, key);
			printf("%s - %s", config->title, (char*)value.dptr);
			free(key.dptr);
			dbm_close(db);
		}else{
			printf("%s - " NIBBLER_NOT_FOUND, config->title);
		}
		free(path);
	}else{
		printf("%s - " NIBBLER_NOT_FOUND, config->title);
	}
	printf("		</title>\n");
	printf("                <link rel=\"stylesheet\" href=\"%s?%s\">\n", config->csspath, date);
	printf("                <link rel=\"icon\" href=\"%s\">\n", config->iconpath);
	printf("	</head>\n");
	printf("	<body>\n");
	printf("		<input type=\"hidden\" value=\"%s\" id=\"prefix\">\n", getenv("SCRIPT_NAME"));
	printf("		<input type=\"hidden\" value=\"%s\" id=\"version\">\n", NIBBLER_VERSION);
	printf("		<div id=\"panel\">\n");
	printf("			<img src=\"%s\" height=\"32px\">\n",  config->iconpath);
	printf("			<span class=\"paneltext\">\n");
	printf("				<button onclick=\"location.href = '%s';\">" NIBBLER_HOME "</button>\n", getenv("SCRIPT_NAME"));
	printf("				<span id=\"buttons\">\n");
	printf("					<button onclick=\"location.href = '%s/login';\" id=\"login-button\">" NIBBLER_LOGIN "</button>\n", getenv("SCRIPT_NAME"));
	if(config->signup == 1){
		printf("					<button onclick=\"location.href = '%s/signup';\" id=\"signup-button\">" NIBBLER_SIGNUP "</button>\n", getenv("SCRIPT_NAME"));
	}
	printf("				</span>\n");
	printf("			</span>");
	printf("		</div>\n");
	printf("		<div id=\"content\">\n");
	if(strcmp(getenv("PATH_REPLACED"), "/") == 0){
		if(config->itworks == 1){
			NIBBLER_WELCOME;
		}else{
			if(config->welcome != NULL){
				FILE* welcome = fopen(config->welcome, "r");
				if(welcome != NULL){
					char ch;
					while(1){
						fread(&ch, 1, 1, welcome);
						if(feof(welcome)){
							break;
						}else{
							printf("%c", ch);
						}
					}
					fclose(welcome);
				}
			}
		}
		printf("<h2>" NIBBLER_THREADS "</h2>\n");
		printf("<div>\n");
		DIR* dir = opendir(NIBBLER_GENRE_DB);
		struct dirent* d;
		while((d = readdir(dir)) != NULL){
			if(strcmp(d->d_name, ".") == 0 || strcmp(d->d_name, "..") == 0) continue;
			struct stat s;
			char* path = sstrcat4(NIBBLER_GENRE_DB, "/", d->d_name, "/description");
			stat(path, &s);
			char* description = malloc(s.st_size + 1);
			description[s.st_size] = 0;
			FILE* f = fopen(path, "r");
			fread(description, s.st_size, 1, f);
			fclose(f);
			free(path);
			path = sstrcat4(NIBBLER_GENRE_DB, "/", d->d_name, "/db");
			printf("<h3>%s - %s</h3>\n", d->d_name, description);
			free(description);
			DBM* db = nibbler_open_db(path);
			datum key;
			key.dptr = strdup("length");
			key.dsize = 7;
			if(dbm_fetch(db, key).dptr == NULL){
				datum value;
				struct nibbler_length len;
				len.type = DB_TYPE_LENGTH;
				len.length = 0;
				value.dptr = (void*)&len;
				value.dsize = sizeof(len);
				dbm_store(db, key, value, DBM_INSERT);
				dbm_close(db);
				db = nibbler_open_db(path);
			}
			free(key.dptr);
			key.dptr = strdup("length");
			key.dsize = 7;
			datum value = dbm_fetch(db, key);
			uint64_t size = ((struct nibbler_length*)value.dptr)->length;
			free(key.dptr);
			printf("<ul>\n");
			key = dbm_firstkey(db);
			while(key.dptr != NULL){
				value = dbm_fetch(db, key);
				char type = ((char*)value.dptr)[0];
				if(type != DB_TYPE_LENGTH){
					struct nibbler_thread* thr = (struct nibbler_thread*)value.dptr;
					char* _t = strdup(thr->title);
					wchar_t* t = malloc(sizeof(wchar_t) * (strlen(_t) + 1));
					t[strlen(_t)] = 0;
					mbstowcs(t, _t, strlen(_t) + 1);
					printf("<li><a href=\"%s/id/%s:%lu\">", getenv("SCRIPT_NAME"), d->d_name, thr->id);
					int i;
					for(i = 0; t[i] != 0 && i < 60; i++){
						if(t[i] == '<'){
							printf("&lt;");
						}else if(t[i] == '>'){
							printf("&gt;");
						}else{
							printf("%lc", t[i]);
						}
					}
					if(strlen(_t) > 60){
						printf("...");
					}
					printf("</a></li>\n");
					free(t);
					free(_t);
				}
				key = dbm_nextkey(db);
			}
			printf("</ul>\n");
			dbm_close(db);
			free(path);
		}
		closedir(dir);
		printf("</div>\n");
	}else if(	strcmp(getenv("PATH_REPLACED"), "/login") == 0
		||	strcmp(getenv("PATH_REPLACED"), "/signup") == 0
		){
		printf("<div style=\"text-align: center;\">\n");
		printf("	<h1>%s</h1>\n", strcmp(getenv("PATH_REPLACED"), "/login") == 0 ? NIBBLER_LOGIN : NIBBLER_SIGNUP);
		printf("	<span class=\"box\">\n");
		printf("		<table border=\"0\">\n");
		printf("			<tr>\n");
		printf("				<th>\n");
		printf("					" NIBBLER_USERNAME "\n");
		printf("				</th>\n");
		printf("				<td>\n");
		printf("					<input id=\"username\">\n");
		printf("				</td>\n");
		printf("			</tr>\n");
		printf("			<tr>\n");
		printf("				<th>\n");
		printf("					" NIBBLER_PASSWORD "\n");
		printf("				</th>\n");
		printf("				<td>\n");
		printf("					<input id=\"password\" type=\"password\">\n");
		printf("				</td>\n");
		printf("			</tr>\n");
		printf("		</table>\n");
		printf("		<button onclick=\"nibbler_%s()\">%s</button>", strcmp(getenv("PATH_REPLACED"), "/login") == 0 ? "login" : "signup", strcmp(getenv("PATH_REPLACED"), "/login") == 0 ? NIBBLER_LOGIN : NIBBLER_SIGNUP);
		printf("	</span>\n");
		printf("</div>\n");
	}else if(strcmp(getenv("PATH_REPLACED"), "/new-thread") == 0){
		printf("<div style=\"text-align: center;\">\n");
		printf("	<h1>" NIBBLER_NEW_THREAD "</h1>\n");
		printf("	<span class=\"box\">\n");
		printf("		<table border=\"0\">\n");
		printf("			<tr>\n");
		printf("				<th>" NIBBLER_NAME "</th>\n");
		printf("				<td><input id=\"thread-name\"></td>\n");
		printf("			</tr>\n");
		printf("			<tr>\n");
		printf("				<th>" NIBBLER_L_GENRE"</th>\n");
		printf("				<td>\n");
		printf("					<select id=\"genre-select\">\n");
		DIR* dir = opendir(NIBBLER_GENRE_DB);
		struct dirent* d;
		while((d = readdir(dir)) != NULL){
			if(strcmp(d->d_name, "..") == 0 || strcmp(d->d_name, ".") == 0) continue;
			printf("<option value=\"%s\">%s</option>\n", d->d_name, d->d_name);
		}
		closedir(dir);
		printf("					</select>\n");
		printf("				</td>\n");
		printf("			</tr>\n");
		printf("		</table>\n");
		printf("		<button onclick=\"nibbler_new_thread()\">" NIBBLER_NEW_THREAD "</button>\n");
		printf("	</span>\n");
		printf("</div>\n");
	}else if(strcmp(getenv("FIRST_PATH"), "/id") == 0){
		if(thread_exists == 1){
			datum key;
			key.dptr = strdup("name");
			key.dsize = 5;
			datum value;
			char* path = sstrcat(NIBBLER_THREAD_DB "/", getenv("PATH_REPLACED") + strlen(getenv("FIRST_PATH")) + 1);
			DBM* db = nibbler_open_db(path);
			if((value = dbm_fetch(db, key)).dptr != NULL){
				printf("<h1>%s</h1><hr>\n", value.dptr);
			}
			free(key.dptr);
			printf("<input type=\"hidden\" value=\"%s\" id=\"id\">\n", getenv("PATH_REPLACED") + strlen(getenv("FIRST_PATH")) + 1);
			uint64_t length = 0;
			key.dptr = strdup("length");
			key.dsize = 7;
			value = dbm_fetch(db, key);
			length = ((struct nibbler_length*)value.dptr)->length;
			uint64_t i;
			free(key.dptr);
			for(i = 0; i < length; i++){
				char* keyname = malloc(1025);
				memset(keyname, 0, 1025);
				sprintf(keyname, "%lu-user", i);
				char* username = NULL;
				key.dptr = keyname;
				key.dsize = strlen(keyname) + 1;
				char* date = NULL;
				if((value = dbm_fetch(db, key)).dptr != NULL){
					username = strdup(value.dptr);
				}
				key.dptr = (void*)&i;
				key.dsize = sizeof(i);
				if((value = dbm_fetch(db, key)).dptr != NULL){
					struct nibbler_message* msg = (struct nibbler_message*)value.dptr;
					struct tm* time_st = localtime(&msg->time);
					char* date1 = malloc(128);
					memset(date1, 0, 128);
					strftime(date1, 128, "%a %b %d %H:%M:%S %Z %Y", time_st);
					time_st = gmtime(&msg->time);
					char* date2 = malloc(128);
					memset(date2, 0, 128);
					strftime(date2, 128, "%a %b %d %H:%M:%S UTC %Y", time_st);
					date = sstrcat3(date1, " | ", date2);
					free(date1);
					free(date2);

				}
				free(keyname);
				printf("<a href=\"#id%lu\" id=\"id%lu\">%lu</a> %s: %s<br>\n", i + 1, i + 1, i + 1, username, date);
				keyname = malloc(1025);
				memset(keyname, 0, 1025);
				sprintf(keyname, "%lu-message", i);
				key.dptr = keyname;
				key.dsize = strlen(keyname) + 1;
				if((value = dbm_fetch(db, key)).dptr != NULL){
					printf("<pre><code class=\"message-data\">");
					wchar_t* str = malloc(sizeof(wchar_t) * (strlen((char*)value.dptr) + 1));
					str[strlen((char*)value.dptr)] = 0;
					mbstowcs(str, (char*)value.dptr, strlen((char*)value.dptr) + 1);
					uint64_t j;
					for(j = 0; str[j] != 0; j++){
						if(str[j] == '<'){
							printf("&lt;");
						}else if(str[j] == '>'){
							printf("&gt;");
						}else{
							printf("%lc", str[j]);
						}
					}
					free(str);
					printf("</code></pre>");
				}
				free(keyname);
				printf("<hr>\n");
				if(username != NULL) free(username);
				if(date != NULL) free(date);
			}
			dbm_close(db);
			free(path);
			printf("<textarea placeholder=\"" NIBBLER_SOMETHING_TO_CHAT "\" style=\"display: none;\" id=\"chat\" disabled></textarea>\n");
			printf("<button id=\"send-button\" style=\"display: none;\" onclick=\"nibbler_send()\" disabled>" NIBBLER_SEND "</button>\n");
		}else{
			printf(NIBBLER_THREAD_NOT_FOUND);
		}
	}else{
		printf("Page not found.\n");
	}
	printf("			<div id=\"bottom\">\n");
	printf("				<span class=\"float-left\">\n");
	printf("					<i>Powered by Nibbler Version: <code>%s %s</code></i>\n", NIBBLER_VERSION, NIBBLER_ID);
	printf("				</span>\n");
	printf("				<span class=\"float-right\">\n");
	printf("					<a href=\"http://sw.e3t.cc/sw/~nishi/nibbler/LICENSE\">" NIBBLER_LICENSE "</a>\n");
	printf("				</span>\n");
	printf("			</div>\n");
	printf("		</div>\n");
	printf("		<script src=\"%s?%s\"></script>\n", config->scriptpath, date);
	printf("	</body>\n");
	printf("</html>\n");
	free(date);
}
